@@ -13,23 +13,57 @@ use Config;
 sub build_binaries {
   my ($self, $build_out, $srcdir) = @_; 
   my $success = 1;    
-  my ($extra_cflags, $extra_lflags) = ('-I/usr/local/include', '-L/usr/local/lib');
-  #if (-d '/usr/X11R6/include' && -d '/usr/X11R6/lib' ) {
-  #  $extra_cflags .= ' -I/usr/X11R6/include';
-  #  $extra_lflags .= ' -L/usr/X11R6/lib';
-  #}
-  #if (-d '/usr/openwin/share/include/' && -d '/usr/openwin/share/lib/') { # solaris 10
-  #  $extra_cflags .= ' -I/usr/openwin/share/include';
-  #  $extra_lflags .= ' -L/usr/openwin/share/lib';
-  #}
-  #if (-d '/opt/local/include' && -d '/opt/local/lib') { #macos
-  #  $extra_cflags .= ' -I/opt/local/include';
-  #  $extra_lflags .= ' -L/opt/local/lib';
-  #}
+  my ($extra_cflags, $extra_lflags) = ('-I/usr/local/include', '-L/usr/local/lib');  
   
-  #x11 - solaris: /usr/openwin/include + /usr/openwin/lib
+  #try to detect some inc/lib directories
+  my $d;
+  $d = $self->run_stdout2str(qw[pkg-config --variable=libdir x11]);
+  my $dir_x11_lib = ($d && -d $d && $d ne '/usr/lib') ? $d : '';
+  $d = $self->run_stdout2str(qw[pkg-config --variable=includedir x11]);
+  my $dir_x11_inc = ($d && -d $d && $d ne '/usr/include') ? $d : '';
+  $d = $self->run_stdout2str(qw[pkg-config --variable=libdir gl]);
+  my $dir_opengl_lib = ($d && -d $d && $d ne '/usr/lib') ? $d : '';
+  $d = $self->run_stdout2str(qw[pkg-config --variable=includedir gl]);
+  my $dir_opengl_inc = ($d && -d $d && $d ne '/usr/include') ? $d : '';
+  $d = $self->run_stdout2str(qw[pkg-config --variable=prefix gtk+-2.0 2]);
+  my $dir_gtk = ($d && -d $d) ? $d : '';
 
+  #platform specific hacks
+  if ($^O eq 'solaris') {
+    $dir_x11_inc    ||= '/usr/dt/include'            if (-f '/usr/dt/include/Xm/Xm.h');
+    $dir_x11_lib    ||= '/usr/dt/lib'                if (-f '/usr/dt/lib/libXm.so');
+    $dir_opengl_inc ||= '/usr/X11/include'           if (-f '/usr/X11/include/GL/gl.h');
+    $dir_opengl_lib ||= '/usr/X11/lib/GL'            if (-f '/usr/X11/lib/GL/libGL.so');
+    $dir_x11_inc    ||= '/usr/openwin/include'       if (-f '/usr/openwin/include/X11/Xlib.h');
+    $dir_x11_lib    ||= '/usr/openwin/lib'           if (-f '/usr/openwin/lib/libX11.so');
+    $dir_x11_inc    ||= '/usr/openwin/share/include' if (-f '/usr/openwin/share/include/X11/Xlib.h');
+    $dir_x11_lib    ||= '/usr/openwin/share/lib'     if (-f '/usr/openwin/share/lib/libX11.so');
+  }
+  
+  if ($^O eq 'darwin') {
+    $extra_cflags .= ' -I/opt/local/include';
+    $extra_lflags .= ' -L/opt/local/lib';
+  }
+
+  #generic /usr/X11R6/...
+  $dir_x11_inc    ||= '/usr/X11R6/include' if (-f '/usr/X11R6/include/X11/Xlib.h');
+  $dir_x11_lib    ||= '/usr/X11R6/lib'     if (-f '/usr/X11R6/lib/libX11.so');
+  $dir_opengl_inc ||= '/usr/X11R6/include' if (-f '/usr/X11R6/include/GL/gl.h');
+  $dir_opengl_lib ||= '/usr/X11R6/lib'     if (-f '/usr/X11R6/lib/libGL.so');
+  #generic /usr/X11R7/...
+  $dir_x11_inc    ||= '/usr/X11R7/include' if (-f '/usr/X11R7/include/X11/Xlib.h');
+  $dir_x11_lib    ||= '/usr/X11R7/lib'     if (-f '/usr/X11R7/lib/libX11.so');
+  $dir_opengl_inc ||= '/usr/X11R7/include' if (-f '/usr/X11R7/include/GL/gl.h');
+  $dir_opengl_lib ||= '/usr/X11R7/lib'     if (-f '/usr/X11R7/lib/libGL.so');
+    
+  $extra_cflags .= " -I$dir_x11_inc"    if $dir_x11_inc;
+  $extra_lflags .= " -L$dir_x11_lib"    if $dir_x11_lib;
+  $extra_cflags .= " -I$dir_opengl_inc" if $dir_opengl_inc;
+  $extra_lflags .= " -I$dir_opengl_lib" if $dir_opengl_lib;
+    
   print "Checking available libraries/headers...\n";
+  print "extra_cflags=$extra_cflags\n";
+  print "extra_lflags=$extra_lflags\n";
   my %has;
 
   $has{gtk}     = `pkg-config --modversion gtk+-2.0 2>/dev/null` ? 1 : 0;        #iupgtk
@@ -61,32 +95,37 @@ sub build_binaries {
   $has{l_glu32} = $self->check_lib( 'glu32' );  # cygwin only
 
   $has{Xm}      = $self->check_header('Xm/Xm.h',   $extra_cflags);
-  $has{Xlib}    = $self->check_header('X11/Xlib.h',$extra_cflags);        #iupgl cdx11
-  $has{glx}     = $self->check_header('GL/glx.h',  $extra_cflags);             #iupgl
+  $has{Xlib}    = $self->check_header('X11/Xlib.h',$extra_cflags); #iupgl cdx11
+  $has{glx}     = $self->check_header('GL/glx.h',  $extra_cflags); #iupgl
   $has{glu}     = $self->check_header('GL/glu.h',  $extra_cflags);
   $has{gl}      = $self->check_header('GL/gl.h',   $extra_cflags);
-  $has{fftw3}   = $self->check_header('fftw3.h',   $extra_cflags);        #im_fftw3 = http://www.fftw.org/
+  $has{fftw3}   = $self->check_header('fftw3.h',   $extra_cflags); #im_fftw3 = http://www.fftw.org/
   $has{windows} = $self->check_header('windows.h');                #iupwin
-  $has{wmsdk}   = $self->check_header('wmsdk.h');                #im_wmv
-  $has{ecw}     = $self->check_header('NCSECWClient.h');        #im_format_ecw = ECW (Enhanced Compression Wavelet) format
-  $has{XxXxX}   = $self->check_header('XxXxX/XxXxX.h');           #non existing header
+  $has{wmsdk}   = $self->check_header('wmsdk.h');                  #im_wmv
+  $has{ecw}     = $self->check_header('NCSECWClient.h');           #im_format_ecw = ECW (Enhanced Compression Wavelet) format
+  $has{XxXxX}   = $self->check_header('XxXxX/XxXxX.h');            #non existing header
 
   print "Has: $has{$_} - $_\n" foreach (sort keys %has);
 
+  #DEBUG ONLY - remove in the release version
   print "Brute force lookup:\n";
-  my $re = qr/\/(Xlib.h|Xm.h|gtk.h|glu.h|gl.h|libX11\.[^\d]*|libGL\.[^\d]*|libXm\.[^\d]*)$/;
+  my $re = qr/\/(Xlib.h|Xm.h|gtk.h|glu.h|glut.h|gl.h|libX11\.[^\d]*|libGL\.[^\d]*|libXm\.[^\d]*)$/;
   print "[/usr    ] $_\n" foreach ($self->find_file('/usr', $re));
+  print "[/lib    ] $_\n" foreach ($self->find_file('/usr', $re));
   print "[/opt    ] $_\n" foreach ($self->find_file('/opt', $re));
   print "[/sw     ] $_\n" foreach ($self->find_file('/sw', $re));
   print "[/System ] $_\n" foreach ($self->find_file('/System', $re));
   print "[/Library] $_\n" foreach ($self->find_file('/Library', $re));
   
   print "Dumping some pkg-info:\n";
-  for my $pkg (qw[gtk+-2.0 gl glu x11 xt xext xmu]) {
+  print "[gtk2 cflags] " . $self->run_stdout2str(qw[pkg-config --cflags gtk+-2.0]) . "\n";
+  print "[gtk2 libs  ] " . $self->run_stdout2str(qw[pkg-config --libs gtk+-2.0]) . "\n";
+  for my $pkg (qw[gtk+-2.0 gl glu glut x11 xt xext xmu]) {
     print "[prefix     $pkg] " . $self->run_stdout2str(qw[pkg-config --variable=prefix], $pkg) . "\n";
     print "[libdir     $pkg] " . $self->run_stdout2str(qw[pkg-config --variable=libdir], $pkg) . "\n";
     print "[includedir $pkg] " . $self->run_stdout2str(qw[pkg-config --variable=includedir], $pkg) . "\n";
   }
+  #DEBUG ONLY - end
 
   my @x11_libs; # just base X11 libs
   push(@x11_libs, 'X11')  if $has{l_X11};
@@ -113,38 +152,49 @@ sub build_binaries {
   $self->config_data('debug_cdtargets', \@cdtargets);
   $self->config_data('debug_iuptargets', \@iuptargets);
 
-  #my @makeopts  = qw[NO_DYNAMIC=Yes USE_NODEPEND=Yes];
-  my @makeopts  = qw[NO_STATIC=Yes USE_NODEPEND=Yes];
+  my @makeopts  = qw[NO_DYNAMIC=Yes USE_NODEPEND=Yes];
+  #my @makeopts  = qw[NO_STATIC=Yes USE_NODEPEND=Yes];
 
   #choose GUI subsystem, priorities if multiple subsystems detected: 1. Win32(cygwin), 2. GTK, 3. X11/Motif
   my @libs;
-  if ($has{xxxwindows}) { #cygwin only
+  if ($has{windows}) { #cygwin only
     push(@makeopts, 'USE_WIN=Yes');
     push(@makeopts, 'X11_LIBS='); #no X11 libs on cygwin
-    push(@libs, qw[gdi32 comdlg32 comctl32 winspool uuid ole32 oleaut32 opengl32 glu32 glut]);
+    push(@libs, qw[gdi32 comdlg32 comctl32 winspool uuid ole32 oleaut32 opengl32 glu32]); #xxx ???glut
     ($extra_cflags, $extra_lflags) = ('', '');
   }
   elsif ($has{gtk}) {
-    my $pref = $self->run_stdout2str(qw[pkg-config --variable=prefix gtk+-2.0 2]);
     push(@makeopts, 'USE_GTK=Yes');
+    #detected libs
+    push(@makeopts, "GTK=$dir_gtk") if $dir_gtk;
     push(@makeopts, "X11_LIBS=" . join(' ', @x11_libs));
+    push(@makeopts, "X11_LIB=$dir_x11_lib") if $dir_x11_lib;
+    push(@makeopts, "X11_INC=$dir_x11_inc") if $dir_x11_inc;
     push(@makeopts, "OPENGL_LIBS=" . join(' ', @opengl_libs));
-    push(@makeopts, "GTK=$pref") if $pref;
+    push(@makeopts, "OPENGL_LIB=$dir_opengl_lib") if $dir_opengl_lib;
+    push(@makeopts, "OPENGL_INC=$dir_opengl_inc") if $dir_opengl_inc;
+ 
     push(@libs, @opengl_libs);
     #Note: $extra_?flags will be stored into ConfigData - they are not used for building
     my @mods = qw[gtk+-2.0 gdk-2.0 pango cairo];
-    $extra_cflags = $self->run_stdout2str(qw[pkg-config --cflags], @mods);
-    $extra_lflags = $self->run_stdout2str(qw[pkg-config --libs], @mods);
+    $extra_cflags = $self->run_stdout2str(qw[pkg-config --cflags], @mods) . " $extra_cflags";
+    $extra_lflags = $self->run_stdout2str(qw[pkg-config --libs], @mods) . " $extra_lflags";
   }
   elsif ($has{Xlib} && $has{Xm}) {
     push(@makeopts, 'USE_X11=Yes');
-    # additional X11 related libs
+    #additional X11 related libs
     push(@x11_libs, 'Xp')   if $has{l_Xp};
     push(@x11_libs, 'Xt')   if $has{l_Xt};
     push(@x11_libs, 'Xm')   if $has{l_Xm};
     push(@x11_libs, 'Xmu')  if $has{l_Xmu};
+    #detected libs
     push(@makeopts, "X11_LIBS=" . join(' ', @x11_libs));
+    push(@makeopts, "X11_LIB=$dir_x11_lib") if $dir_x11_lib;
+    push(@makeopts, "X11_INC=$dir_x11_inc") if $dir_x11_inc;
     push(@makeopts, "OPENGL_LIBS=" . join(' ', @opengl_libs));
+    push(@makeopts, "OPENGL_LIB=$dir_opengl_lib") if $dir_opengl_lib;
+    push(@makeopts, "OPENGL_INC=$dir_opengl_inc") if $dir_opengl_inc;
+
     push(@libs, @x11_libs, @opengl_libs);
     #Note: $extra_?flags set at the beginning of this sub
   }
@@ -189,22 +239,20 @@ sub build_binaries {
   print "Build finished sucessfully!\n";
     
   #DEBUG: fail intentionally here if you want to see build details from cpan testers
-  die "Intentionally failing";
+  #die "Intentionally failing";
 };
 
 sub build_via_tecmake {
   my ($self, $build_out, $srcdir, $mopts, $iuptgs, $cdtgs, $imtgs) = @_;
   $srcdir ||= 'src';
-  my $prefixdir   = rel2abs($build_out);
-  my $make        = $self->get_make;
-  my @makeopts    = @{$mopts};
-  my $makesysinfo = "$make -f tecmake.mak sysinfo MAKENAME= USE_NODEPEND=Yes '" . join("' '", @makeopts) .  "'";
-  my ($im_si, $cd_si, $iup_si);
+  my $prefixdir = rel2abs($build_out);
+  my $make = $self->get_make;
+  my $im_si;
   my $success = 1;
 
   # save it for future use in ConfigData
   $self->config_data('build_prefix', $prefixdir);
-  $self->config_data('debug_makeopts', \@makeopts);
+  $self->config_data('debug_makeopts', $mopts);
 
   #create output directory structure
   mkdir "$prefixdir" unless -d "$prefixdir";
@@ -216,10 +264,10 @@ sub build_via_tecmake {
   if(-d "$srcdir/im/src") {
     print "Gonna build 'im'\n";
     chdir "$srcdir/im/src";
-    ($im_si =`$makesysinfo 2>&1`) =~ s/[\n\r]*$//;
+    $im_si = $self->run_output_tail(undef, $make, qw/-f tecmake.mak sysinfo MAKENAME= USE_NODEPEND=Yes/, @{$mopts});
     print "$im_si\n";
     foreach my $t (@{$imtgs}) {
-      $done{$t} = $self->run_output_tail(1000, $make, $t, @makeopts);
+      $done{$t} = $self->run_output_tail(2000, $make, $t, @{$mopts});
       warn "###WARN### [$?] during make $t" unless $done{$t};
       $success = 0 unless $done{$t};
     }
@@ -231,10 +279,8 @@ sub build_via_tecmake {
   if (-d "$srcdir/cd/src") {
     print "Gonna build 'cd'\n";
     chdir "$srcdir/cd/src";
-    ($cd_si =`$makesysinfo 2>&1`) =~ s/[\n\r]*$//;
-    print "$cd_si\n";
     foreach my $t (@{$cdtgs}) {
-      $done{$t} = $self->run_output_tail(1000, $make, $t, @makeopts);
+      $done{$t} = $self->run_output_tail(2000, $make, $t, @{$mopts});
       warn "###WARN### [$?] during make $t" unless $done{$t};
       $success = 0 unless $done{$t};
     }
@@ -246,10 +292,8 @@ sub build_via_tecmake {
   if (-d "$srcdir/iup") {
     print "Gonna build 'iup'\n";
     chdir "$srcdir/iup";
-    ($iup_si =`$makesysinfo 2>&1`) =~ s/[\n\r]*$//;
-    print "$iup_si\n";
     foreach my $t (@{$iuptgs}) {
-      $done{$t} = $self->run_output_tail(1000, $make, $t, @makeopts);
+      $done{$t} = $self->run_output_tail(2000, $make, $t, @{$mopts});
       warn "###WARN### [$?] during make $t" unless $done{$t};
       $success = 0 unless $done{$t};
     }
@@ -265,7 +309,7 @@ sub build_via_tecmake {
 
   print "Done: $done{$_} - $_\n" foreach (sort keys %done);
   $self->config_data('debug_done', \%done);
-  $self->config_data('debug_si', { im => $im_si, cd => $cd_si, iup => $iup_si } );
+  $self->config_data('debug_si', $im_si);
 
   return $success;
 }
